from op5helpers import *
import os
import subprocess
import textwrap
import yaml

api_endpoint = 'https://'+os.environ.get("MONITOR_SERVER",'localhost')+'/api'
nacoma_api_endpoint = api_endpoint + '/config/'
original_http_api_yml = open('/etc/op5/http_api.yml').read()

def test_can_login_via_nacoma():
    r = requests.get(nacoma_api_endpoint+'host', auth=('monitor', 'monitor'), verify=False)
    eq_(r.status_code, 200)

def test_no_connection_without_auth():
    r = requests.get(nacoma_api_endpoint, verify=False)
    eq_(r.status_code, 401)

def test_no_connection_with_bad_auth():
    r = requests.get(nacoma_api_endpoint, auth=('zumth1ng', 'non-exizt1ng'), verify=False)
    eq_(r.status_code, 401)

def test_correct_http_status_with_bad_request():
    r = requests.get(nacoma_api_endpoint + 'some-junk-here', auth=('monitor', 'monitor'), verify=False)
    eq_(r.status_code, 404)

def restore_conf():
    open('/etc/op5/http_api.yml', 'w').write(original_http_api_yml)

@with_setup(restore_conf, restore_conf)
def test_beta_features_enabled():
    def feature_tester(section, text_sample):
        conf = yaml.load(original_http_api_yml)
        conf['beta_features'] = {section: True}
        with open('/etc/op5/http_api.yml', 'w') as f:
            yaml.dump(conf, f)
        r = requests.get(api_endpoint+'/beta/'+section, auth=('monitor', 'monitor'), verify=False)
        eq_(r.status_code, 200, "Wanted 200, got %(code)d with body:\n%(body)s" % {'code': r.status_code, 'body': r.text})
        ok_(text_sample in r.text, "Could not find\n%s\nin response:\n%s" % (text_sample, r.text))

        r = requests.get(api_endpoint+'/'+section, auth=('monitor', 'monitor'), verify=False)
        eq_(r.status_code, 404, "Wanted 404, got %(code)d with body:\n%(body)s" % {'code': r.status_code, 'body': r.text})
    yield feature_tester, 'perfdata', 'Get the raw performance data'
    yield feature_tester, 'license', 'manage your license for OP5 Monitor'

@with_setup(restore_conf, restore_conf)
def test_beta_features_disabled():
    def feature_tester(section):
        conf = yaml.load(original_http_api_yml)
        conf['beta_features'] = {section: False}
        with open('/etc/op5/http_api.yml', 'w') as f:
            yaml.dump(conf, f)
        r = requests.get(api_endpoint+'/beta/'+section, auth=('monitor', 'monitor'), verify=False)
        eq_(r.status_code, 403, "Wanted 403, got %(code)d with body:\n%(body)s" % {'code': r.status_code, 'body': r.text})
    yield feature_tester, 'perfdata'
    yield feature_tester, 'license'

@with_setup(restore_conf, restore_conf)
def test_auth_method_can_be_disabled():
    locked_down_config = """auth_method:
  ninja: false
  basic_auth: false"""
    open('/etc/op5/http_api.yml', 'w').write(locked_down_config)

    for params in [
            {'auth_method': 'basic_auth'},
            {'auth_method': 'ninja'},
            {}
            ]:
        r = requests.get(nacoma_api_endpoint, auth=('monitor', 'monitor'), params=params, verify=False)
        eq_(r.status_code, 403, "Got %d:\n%s" % (r.status_code, r.text))
        eq_(r.json()['full_error'], "The API is locked down, all 'auth_method's are disabled through http_api.yml", "Got %d:\n%s" % (r.status_code, r.text))

    only_basic_auth_config = """auth_method:
  ninja: false
  basic_auth: true"""
    open('/etc/op5/http_api.yml', 'w').write(only_basic_auth_config)
    r = requests.get(nacoma_api_endpoint, auth=('monitor', 'monitor'), params={'auth_method': 'ninja'}, verify=False)
    eq_(r.status_code, 412, "Got %d:\n%s" % (r.status_code, r.text))
    eq_(r.json()['full_error'], "Given 'auth_method' (ninja) is disabled through http_api.yml, enable it or use another 'auth_method'", "Got %d:\n%s" % (r.status_code, r.text))

class AuthModuleConfig(object):
    def __init__(self):
        self.old_auth = open('/etc/op5/auth.yml').read()
        self.old_authusers = open('/etc/op5/auth_users.yml').read()

    def more_default_modules(self):
        """ @TODO: refactor, since this is a bad idea:
        We're not sure that the fixture below contains the latest and
        greatest needed config options.
        We'd really want to say:
            hey config, erase all old stuff, create *these* modules
        but nacoma's authconfig.php is not a decent enough API for this.

        We could either expose it through the HTTP API, which would let us and
        users access and manipulate config remotely, or we could expose
        authconfig.php's logic via cli.

        The point being: this fixture does not include any business logic
        what so ever - the refactoring must allow us, callers, to not know
        anything about the storage format but provide input parameters and
        have those accepted or not (api 101?)
        """

        new_auth = """common:
  session_key: auth_user
  default_auth: robocop
  enable_auto_login: false
robocop:
  driver: Default
predator:
  driver: Default"""
        new_authusers = """monitor:
  username: monitor
  realname: Monitor Admin
  password: $1$VGn0CdSG$AMJjvHoF8M2nSy8SiPrW70
  modules:
    - Default
    - robocop
    - predator
  groups:
    - admins
  password_algo: crypt"""
        with open('/etc/op5/auth.yml', 'w') as conf:
            conf.write(new_auth)
        with open('/etc/op5/auth_users.yml', 'w') as conf:
            conf.write(new_authusers)

    def header_driver(self):
        yml = """---
common:
  session_key: "auth_user"
  default_auth: "Default"
  enable_auto_login: '1'
  apc_enabled: false
  apc_ttl: ""
  apc_store_prefix: ""
Default:
  driver: "Default"
HeaderAuth:
  driver: "Header"
  header_username: "api_username"
  header_groups: "api_groups"
  header_realname: "api_realname"
  header_email: "api_email"
  group_list_delimiter: ","
"""
        with open('/etc/op5/auth.yml', 'w') as conf:
            conf.write(yml)

    def reset(self):
        with open('/etc/op5/auth.yml', 'w') as conf:
            conf.write(self.old_auth)
        with open('/etc/op5/auth_users.yml', 'w') as conf:
            conf.write(self.old_authusers)

auth_module_config = AuthModuleConfig()
@with_setup(auth_module_config.more_default_modules, auth_module_config.reset)
def test_non_default_auth_module():
    r = requests.get(nacoma_api_endpoint+'host', auth=('monitor$robocop', 'monitor'), verify=False)
    eq_(r.status_code, 200, "Got %d:\n%s\n\n/etc/op5/auth.yml:\n%s\n\n/etc/op5/auth_users.yml:\n%s" % (r.status_code, r.text, open('/etc/op5/auth.yml').read(), open('/etc/op5/auth_users.yml').read()))

    r = requests.get(nacoma_api_endpoint+'host', auth=('monitor$invalid_auth_module', 'monitor'), verify=False)
    eq_(r.status_code, 401, "Got %d:\n%s\n\n/etc/op5/auth.yml:\n%s\n\n/etc/op5/auth_users.yml:\n%s" % (r.status_code, r.text, open('/etc/op5/auth.yml').read(), open('/etc/op5/auth_users.yml').read()))

    r = requests.get(nacoma_api_endpoint+'host', auth=('monitor$predator', 'monitor'), verify=False)
    eq_(r.status_code, 200, "Got %d:\n%s\n\n/etc/op5/auth.yml:\n%s\n\n/etc/op5/auth_users.yml:\n%s" % (r.status_code, r.text, open('/etc/op5/auth.yml').read(), open('/etc/op5/auth_users.yml').read()))

    r = requests.get(nacoma_api_endpoint+'host', auth=('monitor', 'monitor'), verify=False)
    eq_(r.status_code, 200, "Got %d:\n%s\n\n/etc/op5/auth.yml:\n%s\n\n/etc/op5/auth_users.yml:\n%s" % (r.status_code, r.text, open('/etc/op5/auth.yml').read(), open('/etc/op5/auth_users.yml').read()))

def test_neither_header_driver_nor_basic_auth():
    r = requests.get(nacoma_api_endpoint+'host', verify=False)
    eq_(r.status_code, 401, "Should not have been able to auth without credentials, got %d:\n%s" % (r.status_code, r.text))

@with_setup(auth_module_config.header_driver, auth_module_config.reset)
def test_header_driver():
    conforming_headers = {
        'api_username': 'bamse',
        'api_groups': 'admins',
        'api_realname': 'Bamse'
    }
    nonconforming_headers = {
        'lala': 'poe'
    }

    r = requests.get(nacoma_api_endpoint+'host', verify=False, headers=conforming_headers)
    eq_(r.status_code, 200, "Should have been able to auth, got %d:\n%s\nauth.yml:\n%s" % (r.status_code, r.text, open('/etc/op5/auth.yml').read()))

    r = requests.get(nacoma_api_endpoint+'host', verify=False, headers=nonconforming_headers)
    eq_(r.status_code, 401, "Should not have been able to auth, got %d:\n%s\nauth.yml:\n%s" % (r.status_code, r.text, open('/etc/op5/auth.yml').read()))


license_test_dir = '/var/op5license-tests'
license_file = '/etc/op5license/op5license.lic'
op5_license_blaha = license_file + '.blaha'

def _std(**kwargs):
    url = kwargs.get('url', '')
    r = getattr(requests, kwargs.get('method', 'get'))(api_endpoint + url, auth=kwargs.get('user', authed_user), headers=kwargs.get('headers', {}), verify=False)
    if "format=html" not in url:
        try:
            r.json()
        except ValueError:
            ok_(False, "Expected valid JSON, got %d:\n%s" % (r.status_code, r.text))
    code = kwargs.get('code', 200)
    eq_(code, r.status_code, "Wanted status code %d, got %d using %s for URI:\n%s\nbody:\n%s\nheaders:\n%s" % (code, r.status_code, r.request.method, r.url, r.text, r.request.headers))
    return r

def remove_license():
    os.rename(license_file, op5_license_blaha)
    # let's also fake a really old report date so that the license is invalid
    # ... yes, this is ugly because we rely on that we "know" what the current
    # way of mocking a too-old license is
    subprocess.check_output(['mysql', 'merlin', '-e', "INSERT INTO report_data (timestamp, event_type, flags, attrib, host_name, service_description, state, hard, retry, downtime_depth, output, long_output) VALUES (1325030554, 100, NULL, NULL, 'just_a_fake', 'aw_yeah', 0, 0, 0, NULL, NULL, NULL)"])

def put_license_back():
    os.rename(op5_license_blaha, license_file)
    subprocess.check_output(['mysql', 'merlin', '-e', 'DELETE FROM report_data WHERE timestamp = 1325030554 AND host_name = "just_a_fake" AND service_description = "aw_yeah"'])

def install_license(filename):
    subprocess.check_output(['cp', '%(dir)s/signed_valid/%(license)s.lic' % {'dir': license_test_dir, 'license': filename}, license_file])

def setup_mobile():
    remove_license()

def teardown_mobile():
    put_license_back()
    restore_conf()

@with_setup(setup_mobile, teardown_mobile)
def test_mobile_app():
    mobile_user_agents = [
        'op5-Monitor-Mobile-App',
        'op5-Monitor-Mobile-App/1',
        'op5-Monitor-Mobile-App/5.3',
        'op5-Monitor-Mobile-App/dem_monkeys',
    ]
    non_mobile_user_agents = [
        'Mozilla/5.0 (X11; Linux x86_64; rv:35.0) Gecko/20100101 Firefox/35.0'
    ]

    install_license("http_api_only_allow_mobile")

    # Let's not depend on any configuration, combine this url + format=html for
    # a generic true/false operation, regarding the license. Since we're going
    # towards letting the help sections be passwordless, we might as well not
    # use them in this test.
    url = "/filter/query?query=[hosts] all&limit=1"

    has_text = lambda x: ok_(x in r.text, "Missing text '%(search)s' from body:\n%(body)s\nusing the user agent:\n%(ua)s" % {'search': x, 'body': r.text, 'ua': ua})
    not_has_text = lambda x: ok_(x not in r.text, "Found not wanted text '%(search)s' in body:\n%(body)s\nusing the user agent:\n%(ua)s" % {'search': x, 'body': r.text, 'ua': ua})

    for ua in mobile_user_agents:
        r = _std(url=url+"&format=html", code=200, headers={'user-agent': ua, 'x-skip-localhost': '1'})
        has_text('You are viewing the HTML version of this document')
        not_has_text('MOBILE APP WANTED')

    for ua in non_mobile_user_agents:
        r = _std(url=url, code=403, headers={'user-agent': ua, 'x-skip-localhost': '1'})
        has_text('Forbidden')
        has_text('MOBILE APP WANTED')

    install_license("http_api_deny_mobile")

    for ua in mobile_user_agents:
        r = _std(url=url, code=403, headers={'user-agent': ua, 'x-skip-localhost': '1'})
        has_text('Forbidden')
        has_text('MOBILE APP NOT WANTED')

    for ua in non_mobile_user_agents:
        r = _std(url=url+"&format=html", code=200, headers={'user-agent': ua, 'x-skip-localhost': '1'})
        has_text('You are viewing the HTML version of this document')
        not_has_text('MOBILE APP WANTED')

def user_rights_authpoints_setup():
    host_user = Crud('user', 'host_user')
    host_user.post({
        'username': 'host_user',
        'password': 'host_user',
        'modules': ['Default']
    })

    host_contact = Crud('contact', 'host_user')
    host_contact.post({
        'contact_name': 'host_user',
        'alias': 'host_user'
    })

    host_user_group = Crud('usergroup', 'user_host_user')
    # the minimum group rights needed to be allowed in
    host_user_group.post({
        'name': 'user_host_user',
        'access_rights': [
            'host_view_all',
            'host_edit_all',
            'configuration_information',
            'contact_template_view_all',
            'api_status',
            'api_config',
            'hostescalation_view_all',
        ]
    })

    service_user = Crud('user', 'service_user')
    service_user.post({
        'username': 'service_user',
        'password': 'service_user',
        'modules': ['Default']
    })

    service_contact = Crud('contact', 'service_user')
    service_contact.post({
        'contact_name': 'service_user',
        'alias': 'service_user'
    })

    service_user_group = Crud('usergroup', 'user_service_user')
    # the minimum group rights needed to be allowed in
    service_user_group.post({
        'name': 'user_service_user',
        'access_rights': [
            'host_view_all',
            'service_view_all',
            'service_edit_all',
            'configuration_information',
            'api_status',
            'api_config',
            'serviceescalation_view_all',
        ]
    })

    host = Crud('host', 'eine_kleine_host')
    host.post({
        'host_name': 'eine_kleine_host'
    })

    service = Crud('service', 'eine_kleine_host;mayo')
    service.post({
        'host_name': 'eine_kleine_host',
        'service_description': 'mayo',
        'check_command': 'check_merlin_process'
    })
    Change.post()

def user_rights_authpoints_teardown():
    Crud('contact', 'host_user', clean=True).verify_gone()
    Crud('usergroup', 'user_service_user', clean=True).verify_gone()
    Crud('contact', 'service_user', clean=True).verify_gone()
    Crud('user', 'service_user', clean=True).verify_gone()
    Crud('service', 'eine_kleine_host;mayo', clean=True).verify_gone()
    Crud('host', 'eine_kleine_host', clean=True).verify_gone()
    Crud('usergroup', 'user_host_user', clean=True).verify_gone()
    Crud('user', 'host_user', clean=True).verify_gone()
    Change.post()


@with_setup(user_rights_authpoints_setup, user_rights_authpoints_teardown)
def test_user_rights_to_mayi_namespaces_translation():
    # bug #9769 correctly mentions that the 2015.albino-beta1 shortcut
    # that's testing all endpoints with the placeholder resource "hosts"
    # hardcoded, is wrong and should be replaced with the proper object
    # groups
    hu = ('host_user', 'host_user')
    su = ('service_user', 'service_user')
    na = ('nonexisting', 'nonexisting')

    # host user
    _std(url='/filter/query?query=[hosts] all&limit=1', user=hu)
    _std(url='/filter/query?query=[services] all&limit=1', user=hu, code=403)

    _std(url='/status/host/eine_kleine_host', user=hu)
    _std(url='/status/service/eine_kleine_host;mayo', user=hu, code=403)

    _std(url='/config/host/eine_kleine_host', user=hu)
    _std(url='/config/service/eine_kleine_host;mayo', user=hu, code=403)

    _std(url='/config/hostescalation', user=hu)
    _std(url='/config/serviceescalation', user=hu, code=403)

    _std(url='/config/contact_template', user=hu)

    _std(url='/beta/license/data', user=hu, code=403, method='put')
    _std(url='/beta/license/data', user=hu, code=403, method='get')

    # service user
    _std(url='/filter/query?query=[services] all&limit=1', user=su)

    _std(url='/status/service/eine_kleine_host;mayo', user=su)

    _std(url='/config/service/eine_kleine_host;mayo', user=su)

    _std(url='/config/hostescalation', user=su, code=403)
    _std(url='/config/serviceescalation', user=su)

    _std(url='/config/contact_template', user=su, code=403)

    # non user
    _std(url='/filter/query?query=[hosts] all&limit=1', user=na, code=401)
    _std(url='/status/host/eine_kleine_host', user=na, code=401)
    _std(url='/config/host/eine_kleine_host', user=na, code=401)
