<?php

require_once __DIR__.'/../../api/license.php';
require_once __DIR__.'/../../api/interface/permissions_interface.php';
require_once __DIR__.'/../../api/interface/license_interface.php';
require_once __DIR__.'/../../api/interface/logger_interface.php';

class mockLogger implements LoggerInterface {

	private $debug_level;
	private $content;

	public function log($debug_level, $content) {
		$this->debug_level = $debug_level;
		$this->content = $content;
	}

	public function get_debug_level() {
		return $this->debug_level;
	}

	public function get_content() {
		return $this->content;
	}
}

class mockMayI implements PermissionsInterface {

	/**
	 * The $config parameter can contain an element "denied_actions" which
	 * contains a list of namespaces which this instance will deny on invocation
	 * of run()
	 *
	 */
	public function __construct($run_these) {
		$this->run_these = $run_these;
	}

	public function be($context, op5MayI_Actor $actor) {
		return $this;
	}

	public function act_upon(op5MayI_Constraints $constraints, $priority = 0) {
		return $this;
	}

	public function run($action, array $override = array(), &$messages = false, &$metrics = false) {
		$metrics = array();
		$messages = array();

		if(!isset($this->run_these[$action])) {
			throw new Exception("You didn't setup your fixtures properly, tried to run $action");
		}
		if(isset($this->run_these[$action]['metrics'])) {
			$metrics = $this->run_these[$action]['metrics'];
		}
		if(isset($this->run_these[$action]['messages'])) {
			$messages = $this->run_these[$action]['messages'];
		}
		return $this->run_these[$action]['result'];
	}
}

class mockFile {

	private $content = "";
	private $absPath;
	private $written;

	public function __construct($absPath, $written) {
		$this->absPath = $absPath;
		$this->written = $written;
	}

	public function fwrite($content) {
		$this->content = $content;
		return $this->written;
	}

	public function fflush() {}

	public function getRealPath() {
		return $this->absPath;
	}

	public function getWritten() {
		return $this->content;
	}
}

class op5mayimetric {
	private $value;
	private $min;
	private $max;

	public function __construct($value, $min = false, $max = false) {
		$this->value = $value;
		$this->min = $min;
		$this->max = $max;
	}

	public function get_value() {
		return $this->value;
	}

	public function get_min() {
		return $this->min;
	}

	public function get_max() {
		return $this->max;
	}
}

class mockOp5License implements LicenseInterface {

	private $metadata;
	private $valid;

	public function __construct($metadata=null) {
		$this->metadata = $metadata;
	}

	public function get_metadata() {
		return $this->metadata;
	}

	public function is_valid() {}
}

class LicenseTest extends PHPUnit_Framework_TestCase {

	public function op5license_object_data_provider() {
		return array(
			array(
				new mockOp5License(array('test_body_info' => 123)),
				array('test_body_info' => 123)
			),
			array(
				new mockOp5License(array()),
				array()
			),
			array(
				new mockOp5License(null),
				null
			)
		);
	}

	/**
	 * @dataProvider op5license_object_data_provider
	 * @param LicenseInterface $license
	 * @param $expected_metadata
	 */
	public function test_view_get_existing_license(LicenseInterface $license, $expected_metadata) {
		$mayi = new mockMayI(array(
			'monitor.monitoring.hosts:read' => array(
				'metrics' => array(
					'license_time' => new op5mayimetric(1, 1, 1337)
				),
				'result' => true
			)
		));

		$response = License::view($mayi, $license);

		$this->assertInstanceOf('Api_Response', $response);
		$this->assertSame(200, $response->get_code());

		$body = $response->get_body();
		$this->assertArrayHasKey('metadata', $body);
		$this->assertSame($expected_metadata, $body['metadata']);
		$this->assertSame(1337, $body['expires_at']);
	}

	public function test_resource_help_displays() {
		$license = new mockOp5License();
		$baseurl = "testing";

		$response = License::resource_help($baseurl);

		$this->assertInstanceOf('Api_Response', $response);
		$this->assertSame(200, $response->get_code());
		$contents = $response->get_body();
		$this->assertRegExp('/testing\/api\/help\/beta\/license\/data/', $contents['body']);
		$this->assertRegExp('/manage your license/', $contents['body']);
	}

	public function test_data_resource_help_should_display() {
		$license = new mockOp5License();

		$response = License::data_resource_help("");

		$this->assertInstanceOf('Api_Response', $response);
		$this->assertSame(200, $response->get_code());
		$contents = $response->get_body();
		$this->assertRegExp('/Get information about your license/',
			$contents['body']);
	}

	public function test_install() {
		$content = 'testing';
		$file = new mockFile('/path/file', strlen($content));
		$logger = new mockLogger();

		$response = License::install($content, $file, $logger);
		$this->assertInstanceOf('Api_Response', $response);
		$this->assertSame(200, $response->get_code());
		$this->assertSame($content, $file->getWritten());
	}

	public function test_install_nothing_was_written_to_file() {
		$content = 'testing';
		$file = new mockFile('/path/file', 0);
		$logger = new mockLogger();

		$response = License::install($content, $file, $logger);
		$this->assertInstanceOf('Api_Response', $response);
		$this->assertSame(500, $response->get_code());
		$this->assertSame("error", $logger->get_debug_level());
		$this->assertSame("could not write to /path/file. Please make sure that it's owned by apache:apache", $logger->get_content());
	}

}
